\ProvidesClass{qcnp}[2012/02/09]
\LoadClass{article}

% main changes to standard class:
% 
% + author and affiliation DPG-like
% + framebox around abstract

% page layout

\usepackage{geometry}
\geometry{a5paper,
  lmargin=0.5in,
  tmargin=1in,
  rmargin=1in,
  bmargin=1.5in
}

\let\thepage\relax
\pagestyle{headings}
\markright{Abstract submitted to the StoCP 2016 conference}

% fonts

\def\font@title{\large\bf}
\def\font@authors{\normalsize}
\def\font@affiliations{\normalsize\it}

% abstract size: text size minus space used by the frame around the
% text

\newlength\@absheight
\setlength\@absheight\textheight
\advance\@absheight-4\fboxsep
\advance\@absheight-4\fboxrule

\newlength\@abswidth
\setlength\@abswidth\textwidth
\advance\@abswidth-4\fboxsep
\advance\@abswidth-4\fboxrule

% authors and affiliations

% \author gets four arguments: label, affiliation index, first name,
% last name and can occur multiple times.  Each \author command
% increments the counter au@cnt and defines a unique command \au@tsid,
% where id is the value of au@cnt.  This command holds the typeset
% author's name, possibly underlined (if the first argument of \author
% is non-empty), and the affiliation index (argument number 2).  Note
% that we don't verify whether an affiliation actually exists.  Idea
% taken from dpg.sty as used in the LT25 conference.

\newcounter{au@cnt}
\def\author#1#2#3#4{
  \stepcounter{au@cnt}
  \def\test{#1}
  \ifx\@empty\test
    % standard author
    \expandafter\def\csname au@ts\alph{au@cnt}\endcsname{
    #3~\ignorespaces#4\textsuperscript{#2}}
  \else
    % presenting author
    \expandafter\def\csname au@ts\alph{au@cnt}\endcsname{
    \underline{#3~\ignorespaces#4}\textsuperscript{#2}}
  \fi
}

% \@authors is called to do the actual typesetting of the author list

\newcounter{@lp}
\def\@authors{
  % loop over the authors
  \setcounter{@lp}{0}
  \loop
    \stepcounter{@lp}%
    \ifnum\value{@lp}>1\relax
      \ifnum\value{@lp}=\value{au@cnt}\relax
        \ifnum\value{au@cnt}>2\relax
	  % last of more than two authors
	  {, and}%
	\else
	  % last of exactly two authors
	  { and}%
	\fi
      \else
        % next author
        {,}%
      \fi
    % first author, do nothing
    \fi
    {\@nameuse{au@ts\alph{@lp}}}%
    \ifnum\value{@lp}<\value{au@cnt}\relax
  \repeat
}

% \affiliation gets two arguments: the index, the value.  The index is
% used for relating to the author.  As with the \author command, we
% define a counter and a command \aff@tsid, where id is the value of
% aff@cnt.  Note that we don't verify whether an index actually exists
% on the author list.

\newcounter{aff@cnt}
\def\affiliation#1#2{
  \stepcounter{aff@cnt}
  \expandafter\def\csname aff@ts\alph{aff@cnt}\endcsname{%
  \textsuperscript{#1}~\font@affiliations #2}
}

% \@affiliations does the actual typesetting of the affiliations by
% looping over the \aff@tsid commands

\def\@affiliations{%
  % loop over the affiliations
  \setcounter{@lp}{0}%
  \loop
    \stepcounter{@lp}%
    {\@nameuse{aff@ts\alph{@lp}}}
    \ifnum\value{@lp}<\value{aff@cnt}\relax\\
  \repeat
  \vspace{\baselineskip}
}

% references are just typeset as they are, except that a running 
% index is prepended

\newcounter{ref@cnt}
\setcounter{ref@cnt}{0}
\def\reference#1{
  % first reference needs some distance from main text
  \ifnum\value{ref@cnt}=0\relax
    \vspace{.5\baselineskip}
  \fi
  \stepcounter{ref@cnt}
  {\noindent[\arabic{ref@cnt}]~#1\par}
}

% use the abstract environment to actually typeset authors and 
% affiliations

\renewenvironment{abstract}{
  % put the main text into a box
  \newbox\@maintext
  \setbox\@maintext=\vtop\bgroup\hsize\@abswidth
  \noindent{\font@title\@title}\vskip\baselineskip\par
  {\raggedright\font@authors\@authors}\vskip\baselineskip\par
  \noindent\@affiliations\par
  \noindent\ignorespaces
}{
  \par\vfill\egroup
  
  % typeset a warning message if abstract is too long
  \ifdim\dp\@maintext>\@absheight
    \def\@heighterr{\enlargethispage{1.5\baselineskip}
      \vspace{.5\baselineskip}
      \noindent(Abstract exceeds size limit indicated by the box)}
  \else
    \let\@heighterr\relax
  \fi

  % now copy the main text into a framebox with fixed height
  \newbox\@frametext
  \setbox\@frametext=\vsplit\@maintext to \@absheight
  
  % and typeset it with a frame, possibly appending the error message
  \noindent\framebox{\box\@frametext}\par
  \@heighterr
}
